#!/usr/bin/python
#
# Program to generate profiling data.  Run with one argument,
# the profile stats file to generate.

import sys
import os

# Change path so we find Xlib
sys.path.insert(1, os.path.join(sys.path[0], '..'))

from Xlib import X, display, Xatom

import profile

def dostuff():
    d = display.Display()
    r = d.screen().root
    cm = d.screen().default_colormap

    for i in xrange(0, 1000):
        if i % 50 == 0:
            print 'Iteration', i

        r.delete_property(Xatom.WM_NORMAL_HINTS)
        r.delete_property(Xatom.WM_NORMAL_HINTS)
        r.get_geometry()
        r.get_geometry()
        r.delete_property(Xatom.WM_NORMAL_HINTS)
        r.delete_property(Xatom.WM_NORMAL_HINTS)
        r.change_property(Xatom.WM_NORMAL_HINTS, Xatom.STRING, 32, [1, 2, 3, 4])
        r.query_tree()
        cm.query_colors([0, 1, 2, 3, 4, 5, 6, 7])

def main(filename):
    profile.run('dostuff()', filename)

if __name__ == '__main__':
    if len(sys.argv) == 2:
        main(sys.argv[1])
    else:
        print sys.argv[0], "<filename to write profile output to>"



#!/usr/bin/python
#
# $Id: draw-proto.py,v 1.2 2007/06/10 14:11:59 mggrant Exp $
#
# examples/draw.py -- protocol test application.
#
#    Copyright (C) 2000 Peter Liljenberg <petli@ctrl-c.liu.se>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import sys
import os

# Change path so we find Xlib
sys.path.insert(1, os.path.join(sys.path[0], '..'))

from Xlib import X
from Xlib.protocol import display
from Xlib.protocol.request import *

# Application window (only one)
class Window:
    def __init__(self, display):
        self.d = display
        self.objects = []

        # Find which screen to open the window on
        self.screen = self.d.info.roots[self.d.default_screen]

        # Allocate ids to the window and gc
        self.window = self.d.allocate_resource_id()
        self.gc = self.d.allocate_resource_id()

        # Create a window
        CreateWindow(self.d, None,
                     self.screen.root_depth,
                     self.window,
                     self.screen.root,
                     50, 50, 300, 200, 2,
                     X.InputOutput,
                     X.CopyFromParent,

                     # special attribute values
                     background_pixel = self.screen.white_pixel,
                     event_mask = (X.ExposureMask |
                                   X.StructureNotifyMask |
                                   X.ButtonPressMask |
                                   X.ButtonReleaseMask |
                                   X.Button1MotionMask),
                     colormap = X.CopyFromParent)

        # Create a gc for drawing
        CreateGC(self.d, None,
                 self.gc,
                 self.window,

                 # special attribute values
                 foreground = self.screen.black_pixel,
                 background = self.screen.white_pixel)

        # Map the window, making it visible
        MapWindow(self.d, None, self.window)

    # Main loop, handling events

    def loop(self):
        current = None
        while 1:
            e = self.d.next_event()

            # Window has been destroyed, quit
            if e.type == X.DestroyNotify:
                sys.exit(0)

            # Some part of the window has been exposed,
            # redraw all the objects.
            if e.type == X.Expose:
                for o in self.objects:
                    o.expose(e)

            # Left button pressed, start to draw
            if e.type == X.ButtonPress and e.detail == 1:
                current = Movement(self, e)
                self.objects.append(current)

            # Left button released, finish drawing
            if e.type == X.ButtonRelease and e.detail == 1 and current:
                current.finish(e)
                current = None

            # Mouse movement with button pressed, draw
            if e.type == X.MotionNotify and current:
                current.motion(e)

# A drawed objects, consisting of either a single
# romboid, or two romboids connected by a winding line

class Movement:
    def __init__(self, win, ev):
        self.win = win

        self.left = ev.event_x - 5
        self.right = ev.event_x + 5
        self.top = ev.event_y - 5
        self.bottom = ev.event_y + 5

        self.time = ev.time
        self.lines = [(ev.event_x, ev.event_y)]

        self.first = Romboid(self.win, ev)
        self.last = None

    def motion(self, ev):
        # Find all the mouse coordinates since the
        # last event received

        r = GetMotionEvents(self.win.d,
                            window = self.win.window,
                            start = self.time,
                            stop = ev.time)

        if not r.events:
            return

        # Record the previous last coordinate, and append
        # the new coordinates

        firstline = len(self.lines) - 1

        # Discard the first coordinate if that is identical to
        # the last recorded coordinate

        pos = r.events[0]
        if (pos.x, pos.y) == self.lines[-1]:
            events = r.events[1:]
        else:
            events = r.events

        # Append all coordinates
        for pos in events:
            x = pos.x
            y = pos.y

            if x < self.left:
                self.left = x
            if x > self.right:
                self.right = x

            if y < self.top:
                self.top = y
            if y > self.bottom:
                self.bottom = y

            self.lines.append((x, y))

        # Append the event coordinate, if that is different from the
        # last movement coordinate
        if (ev.event_x, ev.event_y) != self.lines[-1]:
            self.lines.append((ev.event_x, ev.event_y))

        # Draw a line between the new coordinates
        PolyLine(self.win.d, None,
                 X.CoordModeOrigin,
                 self.win.window,
                 self.win.gc,
                 self.lines[firstline:])

        self.time = ev.time

    def finish(self, ev):
        self.motion(ev)
        if len(self.lines) > 1:
            self.last = Romboid(self.win, ev)

            self.left = min(ev.event_x - 5, self.left)
            self.right = max(ev.event_x + 5, self.right)
            self.top = min(ev.event_y - 5, self.top)
            self.bottom = max(ev.event_y + 5, self.bottom)

    def expose(self, ev):
        # We should check if this object is in the exposed
        # area, but I can't be bothered right now, so just
        # redraw on the last Expose in every batch

        if ev.count == 0:
            self.first.draw()
            if self.last:
                # Redraw all the lines
                PolyLine(self.win.d, None,
                         X.CoordModeOrigin,
                         self.win.window,
                         self.win.gc,
                         self.lines)
                self.last.draw()


# A romboid, drawed around the Movement endpoints
class Romboid:
    def __init__(self, win, ev):
        self.win = win
        self.x = ev.event_x
        self.y = ev.event_y
        self.draw()

    def draw(self):
        # Draw the segments of the romboid
        PolyLine(self.win.d, None,
                 X.CoordModePrevious,
                 self.win.window,
                 self.win.gc,
                 [(self.x, self.y - 5),
                  (5, 5),
                  (-5, 5),
                  (-5, -5),
                  (5, -5)])


if __name__ == '__main__':
    Window(display.Display()).loop()



#!/usr/bin/env python

import sys
import os

sys.path.insert(1, os.path.join(sys.path[0], '../..'))

import types
import string
import struct
from whrandom import randint, choice

from Xlib.protocol import request, structs, rq, event
from Xlib import X, XK

MINI_DEF = (('CARD8', 'reqType'),
            ('BYTE', 'pad'),
            ('CARD16', 'length'))

RESOURCE_DEF = (('CARD8', 'reqType'),
                ('BYTE', 'pad'),
                ('CARD16', 'length'),
                ('CARD32', 'id'))

def read_defs():
    global request_defs, reply_defs, struct_defs
    global mini_request_defs, resource_request_defs
    global event_defs

    request_defs = {}
    mini_request_defs = {}
    resource_request_defs = {}
    reply_defs = {}
    struct_defs = {}
    event_defs = {}

    for line in sys.stdin.readlines():
        parts = string.split(string.strip(line))

        fields = []
        for f in parts[2:]:
            fields.append(string.split(f, ':'))

        if parts[0] == 'REQUEST':
            request_defs[parts[1]] = fields
        elif parts[0] == 'MINIREQUEST':
            mini_request_defs[parts[1]] = MINI_DEF
        elif parts[0] == 'RESOURCEREQUEST':
            resource_request_defs[parts[1]] = RESOURCE_DEF
        elif parts[0] == 'REPLY':
            reply_defs[parts[1]] = fields
        elif parts[0] == 'STRUCT':
            struct_defs[parts[1]] = fields
        elif parts[0] == 'EVENT':
            event_defs[parts[1]] = fields


def build():
    if struct.unpack('BB', struct.pack('H', 0x0100))[0]:
        endian = 'be'
    else:
        endian = 'le'

    read_defs()

    build_request(endian)
    build_event(endian)


def build_request(endian):
    fc = open('genrequest.c', 'w')

    fc.write(C_HEADER)

    reqlist = request.major_codes.items()
    reqlist.sort(lambda x, y: cmp(x[0], y[0]))

    genfuncs = []
    req_args = {}
    reply_args = {}

    for code, req in reqlist:
        name = req.__name__
        creqname = name

        cdefs = request_defs.get(name)
        if cdefs is None:
            cdefs = mini_request_defs.get(name)
            creqname = ''
        if cdefs is None:
            cdefs = resource_request_defs.get(name)
            creqname = 'Resource'

        creqname = 'x%sReq' % creqname

        if cdefs is None:
            sys.stderr.write('missing def for request: %s\n' % name)
        else:
            vardefs = request_var_defs.get(name, [()])
            if type(vardefs) is not types.ListType:
                vardefs = [vardefs]

            i = 0
            for v in vardefs:
                if i > 0:
                    uname = name + str(i)
                else:
                    uname = name

                try:
                    req_args[uname] = gen_func(fc,
                                              'genrequest_' + uname,
                                              creqname,
                                              'REQUEST ' + uname,
                                              req._request,
                                              cdefs,
                                              v)
                except:
                    sys.stderr.write('Error in %s request\n' % uname)
                    raise

                genfuncs.append('genrequest_' + uname)
                i = i + 1

        if issubclass(req, rq.ReplyRequest):
            cdefs = reply_defs.get(name)

            if cdefs is None:
                sys.stderr.write('missing def for reply: %s\n' % name)
            else:
                vardefs = reply_var_defs.get(name, ())
                if type(vardefs) is not types.ListType:
                    vardefs = [vardefs]

                i = 0
                for v in vardefs:
                    if i > 0:
                        uname = name + str(i)
                    else:
                        uname = name

                    try:
                        reply_args[uname] = gen_func(fc,
                                                     'genreply_' + uname,
                                                     'x%sReply' % name,
                                                     'REPLY ' + uname,
                                                     req._reply,
                                                     cdefs,
                                                     v)
                    except:
                        sys.stderr.write('Error in %s reply\n' % uname)
                        raise

                    genfuncs.append('genreply_' + uname)
                    i = i + 1


    fc.write('''

    int main(void)
    {
    ''')

    for gf in genfuncs:
        fc.write('      %s();\n' % gf)

    fc.write('''
      return 0;
    }
    ''')

    fc.close()
    os.system('gcc -Wall -g genrequest.c -o genrequest')

    req_bins = {}
    reply_bins = {}
    pc = os.popen('./genrequest', 'r')
    for line in pc.readlines():
        parts = string.split(string.strip(line))
        if parts[0] == 'REQUEST':
            req_bins[parts[1]] = parts[2]
        elif parts[0] == 'REPLY':
            reply_bins[parts[1]] = parts[2]

    fpy = open('test_requests_%s.py' % endian, 'w')
    os.chmod('test_requests_%s.py' % endian, 0755)

    if endian == 'be':
        e = 'Big-endian'
        v = 1
    else:
        e = 'Little-endian'
        v = 0

    fpy.write(PY_HEADER % { 'endname': e, 'endvalue': v })

    for code, req in reqlist:
        name = req.__name__

        fpy.write('\n\nclass Test%s(unittest.TestCase):\n' % name)
        fpy.write('    def setUp(self):\n')

        i = 0
        reqs = -1
        replies = -1
        while 1:
            if i > 0:
                uname = name + str(i)
            else:
                uname = name

            reqbin = req_bins.get(uname)
            replybin = reply_bins.get(uname)

            if reqbin is None and replybin is None:
                break

            if reqbin:
                reqs = i
                fpy.write('        self.req_args_%d = %s\n'
                          % (i, build_args(req_args[uname])))
                fpy.write('        self.req_bin_%d = %s\n\n'
                          % (i, build_bin(reqbin)))
            if replybin:
                replies = i
                fpy.write('        self.reply_args_%d = %s\n'
                          % (i, build_args(reply_args[uname])))
                fpy.write('        self.reply_bin_%d = %s\n\n'
                          % (i, build_bin(replybin)))

            i = i + 1

        for i in range(0, reqs + 1):
            fpy.write('''
    def testPackRequest%(n)d(self):
        bin = apply(request.%(req)s._request.to_binary, (), self.req_args_%(n)d)
        self.assert_(bin == self.req_bin_%(n)d, tohex(bin))

    def testUnpackRequest%(n)d(self):
        args, remain = request.%(req)s._request.parse_binary(self.req_bin_%(n)d, dummy_display, 1)
        self.assert_(len(remain) == 0, tohex(remain))
        self.assert_(args == self.req_args_%(n)d, args)
''' % { 'req': req.__name__, 'n': i })

        for i in range(0, replies + 1):
            fpy.write('''
    def testPackReply%(n)d(self):
        bin = apply(request.%(req)s._reply.to_binary, (), self.reply_args_%(n)d)
        self.assert_(bin == self.reply_bin_%(n)d, tohex(bin))

    def testUnpackReply%(n)d(self):
        args, remain = request.%(req)s._reply.parse_binary(self.reply_bin_%(n)d, dummy_display, 1)
        self.assert_(len(remain) == 0, tohex(remain))
        self.assert_(args == self.reply_args_%(n)d, args)
''' % { 'req': req.__name__, 'n': i })

    fpy.write('''

if __name__ == "__main__":
    check_endian()
    unittest.main()
''')


def build_event(endian):
    fc = open('genevent.c', 'w')

    fc.write(C_HEADER)

    evtlist = event.event_class.items()
    evtlist.sort(lambda x, y: cmp(x[0], y[0]))

    genfuncs = []
    evt_args = {}

    for code, evt in evtlist:

        # skip events that does not subclass rq.Event immediately,
        # since those are specializations of the more general ones we
        # test.

        if evt.__bases__ != (rq.Event, ):
            continue

        # special handling of KeymapNotify, since that
        # event is so different
        if evt == event.KeymapNotify:
            evt_args['KeymapNotify'] = gen_func(fc,
                                                'genevent_KeymapNotify',
                                                'xKeymapEvent',
                                                'EVENT KeymapNotify',
                                                evt._fields,
                                                (('BYTE', 'type'), ),
                                                (31, ))
            genfuncs.append('genevent_KeymapNotify')
            continue

        name = evt.__name__

        cdefs = event_defs.get(name)
        if cdefs is None:
            sys.stderr.write('missing def for event: %s\n' % name)
        else:
            vardefs = event_var_defs.get(name, [()])
            if type(vardefs) is not types.ListType:
                vardefs = [vardefs]

            i = 0
            for v in vardefs:
                if i > 0:
                    uname = name + str(i)
                else:
                    uname = name

                try:
                    evt_args[uname] = gen_func(fc,
                                               'genevent_' + uname,
                                               'xEvent',
                                               'EVENT ' + uname,
                                               evt._fields,
                                               cdefs,
                                               v)
                except:
                    sys.stderr.write('Error in %s event\n' % uname)
                    raise

                genfuncs.append('genevent_' + uname)
                i = i + 1

    fc.write('''

    int main(void)
    {
    ''')

    for gf in genfuncs:
        fc.write('      %s();\n' % gf)

    fc.write('''
      return 0;
    }
    ''')

    fc.close()
    os.system('gcc -Wall -g genevent.c -o genevent')

    evt_bins = {}
    pc = os.popen('./genevent', 'r')
    for line in pc.readlines():
        parts = string.split(string.strip(line))
        if parts[0] == 'EVENT':
            evt_bins[parts[1]] = parts[2]

    fpy = open('test_events_%s.py' % endian, 'w')
    os.chmod('test_events_%s.py' % endian, 0755)

    if endian == 'be':
        e = 'Big-endian'
        v = 1
    else:
        e = 'Little-endian'
        v = 0

    fpy.write(PY_HEADER % { 'endname': e, 'endvalue': v })

    for code, evt in evtlist:
        if evt.__bases__ != (rq.Event, ):
            continue

        name = evt.__name__

        fpy.write('\n\nclass Test%s(unittest.TestCase):\n' % name)
        fpy.write('    def setUp(self):\n')

        i = 0
        evts = -1
        while 1:
            if i > 0:
                uname = name + str(i)
            else:
                uname = name

            evtbin = evt_bins.get(uname)

            if evtbin is None:
                break

            evts = i
            fpy.write('        self.evt_args_%d = %s\n'
                      % (i, build_args(evt_args[uname])))
            fpy.write('        self.evt_bin_%d = %s\n\n'
                      % (i, build_bin(evtbin)))
            i = i + 1

        for i in range(0, evts + 1):
            fpy.write('''
    def testPack%(n)d(self):
        bin = apply(event.%(evt)s._fields.to_binary, (), self.evt_args_%(n)d)
        self.assert_(bin == self.evt_bin_%(n)d, tohex(bin))

    def testUnpack%(n)d(self):
        args, remain = event.%(evt)s._fields.parse_binary(self.evt_bin_%(n)d, dummy_display, 1)
        self.assert_(len(remain) == 0, tohex(remain))
        self.assert_(args == self.evt_args_%(n)d, args)
''' % { 'evt': evt.__name__, 'n': i })

    fpy.write('''

if __name__ == "__main__":
    check_endian()
    unittest.main()
''')


def gen_func(fc, funcname, structname, outputname, pydef, cdef, vardefs):
    fc.write('''void %s(void)
    {
      struct {
        %s xstruct;
        ''' % (funcname, structname))

    args = {}
    varfs = {}
    extra_vars = []
    flags = None

    # structure fields etc
    i = 0
    for f in pydef.var_fields:
        #
        # List of something
        #
        if isinstance(f, rq.List):
            #
            # List of short strings
            #
            if f.type is rq.Str:
                vfstrings = vardefs[i]
                vflen = 0
                vfdata = ''
                for s in vfstrings:
                    vflen = vflen + 1 + len(s)
                    vfdata = vfdata + chr(len(s)) + s

                fc.write('unsigned char %s[%d];\n      '
                         % (f.name, pad4(vflen)))
                varfs[f.name] = ('memcpy(data.%s, %s, %d);'
                                 % (f.name, cstring(vfdata), vflen),
                                 len(vfstrings), 0)
                args[f.name] = vfstrings

            #
            # List of scalars
            #
            elif isinstance(f.type, rq.ScalarObj) \
                 or isinstance(f.type, rq.ResourceObj):

                vflen = vardefs[i]

                if f.type.structcode == 'B':
                    rmin = 128
                    rmax = 255
                    deflen = pad4(vflen)
                    ctype = 'CARD8'
                elif f.type.structcode == 'H':
                    rmin = 32768
                    rmax = 65536
                    deflen = vflen + vflen % 2
                    ctype = 'CARD16'
                elif f.type.structcode == 'L':
                    rmin = 65536
                    rmax = 2147483646
                    deflen = vflen
                    ctype = 'CARD32'
                else:
                    RuntimeError('oops: %s' % f.type.structcode)

                def rand(x, rmin = rmin, rmax = rmax):
                    return randint(rmin, rmax)

                vfdata = map(rand, range(0, vflen))

                #
                # Special case for a few in-line coded lists
                #
                if structname in ('xGetKeyboardControlReply',
                                  'xQueryKeymapReply',
                                  'xKeymapEvent'):
                    extra_vars.append('%s %s_def[%d] = { %s };'
                                      % (ctype, f.name, vflen,
                                         string.join(map(str, vfdata), ', ')))
                    varfs[f.name] = ('memcpy(data.xstruct.map, %s_def, sizeof(%s_def));'
                                     % (f.name, f.name),
                                     vflen, 0)
                else:
                    fc.write('%s %s[%d];\n      '
                             % (ctype, f.name, deflen))
                    extra_vars.append('%s %s_def[%d] = { %s };'
                                      % (ctype, f.name, vflen,
                                         string.join(map(str, vfdata), ', ')))
                    varfs[f.name] = ('memcpy(data.%s, %s_def, sizeof(%s_def));'
                                     % (f.name, f.name, f.name),
                                     vflen, 0)

                args[f.name] = vfdata

            #
            # Special handling of unique Host case
            #
            elif f.type is structs.Host:
                pydata = [{ 'family': X.FamilyInternet,
                            'name': [ 34, 23, 178, 12 ] },
                          { 'family': X.FamilyInternet,
                            'name': [ 130, 236, 254, 15 ] }, ]

                cdata = []
                for p in pydata:
                    cdata.append("{ { %d, 0, 4 }, { %d, %d, %d, %d } }"
                                 % ((p['family'], ) + tuple(p['name'])))

                fc.write('struct { xHostEntry e; CARD8 name[4]; } %s[2];\n      ' % f.name)

                extra_vars.append('struct { xHostEntry e; CARD8 name[4]; } %s_def[%d] = { %s };'
                                  % (f.name, len(pydata),
                                     string.join(cdata, ', ')))

                varfs[f.name] = ('memcpy(data.%s, %s_def, sizeof(%s_def));'
                                 % (f.name, f.name, f.name),
                                 len(pydata), 0)

                args[f.name] = pydata


            #
            # List of structures
            #
            elif isinstance(f.type, rq.Struct):
                vfname, vflen = vardefs[i]
                vfdef = struct_defs[vfname]

                pydata = []
                defdata = []
                for si in range(0, vflen):
                    d = []
                    for t, cf in vfdef:
                        if cf[:3] != 'pad':
                            d.append(gen_value(t))

                    pyd = {}
                    for sj in range(0, len(d)):
                        pyd[f.type.fields[sj].name] = d[sj]

                    pydata.append(pyd)
                    defdata.append('{ ' + string.join(map(str, d), ', ') + ' }')

                fc.write('x%s %s[%d];\n        ' % (vfname, f.name, vflen))

                extra_vars.append('x%s %s_def[%d] = { %s };'
                                  % (vfname, f.name, vflen,
                                     string.join(defdata, ', ')))
                varfs[f.name] = ('memcpy(data.%s, %s_def, sizeof(%s_def));'
                                 % (f.name, f.name, f.name),
                                 vflen, 0)
                args[f.name] = pydata

        #
        # wide-char string
        #
        elif isinstance(f, rq.String16):
            vfstr = vardefs[i]
            vflen = len(vfstr)

            fc.write('unsigned char %s[%d];\n        ' %
                     (f.name, (vflen + vflen % 2) * 2))

            s = ''
            for c in vfstr:
                s = s + '\0' + c

            varfs[f.name] = ('memcpy(data.%s, %s, %d);'
                             % (f.name, cstring(s), vflen * 2),
                             vflen, 0)
            args[f.name] = tuple(map(ord, vfstr))

        #
        # byte-char string
        #
        elif isinstance(f, rq.String8):
            vfstr = vardefs[i]
            vflen = len(vfstr)

            fc.write('unsigned char %s[%d];\n        ' %
                     (f.name, (vflen + (4 - vflen % 4) % 4)))
            varfs[f.name] = ('memcpy(data.%s, %s, %d);'
                             % (f.name, cstring(vfstr), vflen),
                             vflen, 0)
            args[f.name] = vfstr

        #
        # List of optional values
        #
        elif isinstance(f, rq.ValueList):
            vlcode = []
            vlargs = {}
            flags = 0
            for vlf, flag in f.fields:
                ctype, rmin, rmax, clen = structcode_defs[vlf.structcode]
                fc.write('%s %s_%s;\n      '
                         % (ctype, f.name, vlf.name))
                if clen < 4:
                    fc.write('unsigned char %s_%s_pad[%d];\n      '
                             % (f.name, vlf.name, 4 - clen))

                if isinstance(vlf, rq.Set):
                    val = choice(vlf.values)
                elif isinstance(vlf, rq.Bool):
                    val = choice((0, 1))
                else:
                    val = randint(rmin, rmax)
                vlargs[vlf.name] = val
                vlcode.append('data.%s_%s = %d;' % (f.name, vlf.name, val))
                flags = flags | flag

            # vlcode.append('data.%s_flags = %d;' % (f.name, flags))

            varfs[f.name] = (string.join(vlcode, ' '), 0, 0)
            args[f.name] = vlargs

        #
        # Text/font list, hardwire since they are so rare
        #
        elif isinstance(f, rq.TextElements8):
            if isinstance(f, rq.TextElements16):
                vfstr = '\x02\x02\x10\x23\x00\x12\xff\x01\x02\x03\x04'
                ret = [{'delta': 2, 'string': (0x1023, 0x0012)},
                       0x01020304]
            else:
                vfstr = '\x03\x02zoo\xff\x01\x02\x03\x04\x02\x00ie'
                ret = [{'delta': 2, 'string': 'zoo'},
                       0x01020304,
                       { 'delta': 0, 'string': 'ie'}]

            fc.write('unsigned char %s[%d];\n      '
                     % (f.name, len(vfstr)))
            varfs[f.name] = ('memcpy(data.%s, %s, %d);'
                             % (f.name, cstring(vfstr), len(vfstr)),
                             0, 0)
            args[f.name] = ret

        #
        # Keyboard/modifier mappings
        #
        elif isinstance(f, rq.KeyboardMapping) \
             or isinstance(f, rq.ModifierMapping):

            if isinstance(f, rq.KeyboardMapping):
                rmin = 0
                rmax = 2147483646
                length = 20
                format = 3
                ctype = 'CARD32'
            else:
                rmin = 0
                rmax = 255
                length = 8
                format = 2
                ctype = 'CARD8'

            pydata = []
            cdata = []
            for i in range(0, length):
                x = []
                for j in range(0, format):
                    v = randint(rmin, rmax)
                    x.append(v)
                    cdata.append(str(v))
                pydata.append(x)

            fc.write('%s %s[%d];\n      ' % (ctype, f.name, len(cdata)))
            extra_vars.append('%s %s_def[%d] = { %s };'
                              % (ctype, f.name, len(cdata),
                                 string.join(cdata, ', ')))
            varfs[f.name] = ('memcpy(data.%s, %s_def, sizeof(%s_def));'
                             % (f.name, f.name, f.name),
                             length, format)
            args[f.name] = pydata

        #
        # property data
        #
        elif isinstance(f, rq.PropertyData):
            format, data = vardefs[i]
            length = len(data)

            if format == 8:
                ctype = 'CARD8'
                clen = pad4(length)
                cdata = cstring(data)
            elif format == 16:
                ctype = 'CARD16'
                clen = length + length % 2
                cdata = string.join(map(str, data), ', ')
            elif format == 32:
                ctype = 'CARD32'
                clen = length
                cdata = string.join(map(str, data), ', ')

            if not isinstance(f, rq.FixedPropertyData):
                fc.write('%s %s[%d];\n        ' %
                         (ctype, f.name, clen))

            extra_vars.append('%s %s_def[%d] = { %s };'
                              % (ctype, f.name, length, cdata))

            if not isinstance(f, rq.FixedPropertyData):
                varfs[f.name] = ('memcpy(data.%s, %s_def, sizeof(%s_def));'
                                 % (f.name, f.name, f.name),
                                 length, format)
            else:
                varfs[f.name] = ('assert(sizeof(%s_def) == 20); memcpy(data.xstruct.u.clientMessage.u.b.bytes, %s_def, sizeof(%s_def));'
                                 % (f.name, f.name, f.name),
                                 length, format)

            args[f.name] = (format, data)

        #
        # Event
        #
        elif isinstance(f, rq.EventField):
            ev = event.Expose(window = gen_value('CARD32'),
                              x = gen_value('CARD16'),
                              y =  gen_value('CARD16'),
                              width = gen_value('CARD16'),
                              height = gen_value('CARD16'),
                              count = gen_value('CARD16'))
            cdata = cstring(ev._binary)

            # fc.write('unsigned char %s[32];\n        ' % f.name)
            extra_vars.append('unsigned char %s_def[32] = %s;'
                              % (f.name, cdata))
            varfs[f.name] = ('memcpy(&data.xstruct.event, %s_def, sizeof(%s_def));'
                             % (f.name, f.name),
                             0, 0)

            args[f.name] = ev

        else:
            raise RuntimeError('oops: %s.%s' % (funcname, f.name))

        i = i + 1


    fc.write('\n      } data;\n')


    for v in extra_vars:
        fc.write('      %s\n' % v)

    fc.write('''
      memset(&data, 0, sizeof(data));

    ''')

    pyi = 0
    ci = 0

    while ci < len(cdef):
        try:
            pyf = pydef.fields[pyi]
        except IndexError:
            pyf = None

        cf = cdef[ci]
        t, f = cf

        pyi = pyi + 1
        ci = ci + 1

        if f[:3] == 'pad' or f[:6] == 'walign':
            if not isinstance(pyf, rq.Pad):
                pyi = pyi - 1

        # special case for value list mask
        elif (f == 'mask' or f == 'valueMask') and flags is not None:
            fc.write('      data.xstruct.%s = %d;\n' % (f, flags))

        elif isinstance(pyf, rq.ConstantField):
            fc.write('      data.xstruct.%s = %d;\n' % (f, pyf.value))

        elif isinstance(pyf, rq.RequestLength):
            assert f == 'length'

            fc.write('      assert(sizeof(data) % 4 == 0);\n')
            fc.write('      data.xstruct.length = sizeof(data) / 4;\n')

        elif isinstance(pyf, rq.ReplyLength):
            assert f == 'length'

            fc.write('      assert(sizeof(data) % 4 == 0);\n')
            fc.write('      assert(sizeof(data) >= 32);\n')
            fc.write('      data.xstruct.length = (sizeof(data) - 32) / 4;\n')

        elif isinstance(pyf, rq.LengthOf):
            fc.write('      data.xstruct.%s = %d;\n' % (f, varfs[pyf.name][1]))

        elif isinstance(pyf, rq.OddLength):
            fc.write('      data.xstruct.%s = %d;\n' % (f, varfs[pyf.name][1] % 2))

        elif isinstance(pyf, rq.Format):
            fc.write('      data.xstruct.%s = %d;\n' % (f, varfs[pyf.name][2]))

        elif isinstance(pyf, rq.Set):
            val = choice(pyf.values)
            fc.write('      data.xstruct.%s = %d;\n' % (f, val))
            args[pyf.name] = val

        elif t == 'xCharInfo':
            d = []
            for ct, cf in struct_defs['CharInfo']:
                if cf[:3] != 'pad':
                    d.append(gen_value(ct))

            pyd = {}
            for sj in range(0, len(d)):
                pyd[pyf.type.fields[sj].name] = d[sj]

            fc.write('{ %s def = { %s };\n      '
                     % (t, string.join(map(str, d), ', ')))
            fc.write('memcpy(&data.xstruct.%s, &def, sizeof(def)); }\n        ' % f)
            args[pyf.name] = pyd

        else:
            val = gen_value(t)
            fc.write('      data.xstruct.%s = %d;\n' % (f, val))
            args[pyf.name] = val

    for code, length, format in varfs.values():
        fc.write('      %s\n' % code);

    fc.write('''
      output("%s", &data, sizeof(data));
    }

    ''' % outputname)

    return args


def gen_value(t):
    if t == 'INT8':
        val = randint(-128, -1)
    elif t == 'INT16':
        val = randint(-32768, -256)
    elif t == 'INT32':
        val = randint(-2147483647, -65536)
    elif t == 'CARD8' or t == 'BYTE':
        val = randint(128, 255)
    elif t == 'CARD16':
        val = randint(256, 65535)
    elif t == 'CARD32':
        val = randint(65536, 2147483646)
    elif t == 'BOOL':
        val = randint(0, 1)
    else:
        raise RuntimeError('unknown type: %s' % t)
    return val


def pad4(l):
    return l + (4 - l % 4) % 4

def cstring(s):
    return '"' + string.join(map(lambda c: '\\x%x' % ord(c), s), '') + '"'


def build_args(args):
    kwlist = []
    for kw, val in args.items():
        kwlist.append("            '%s': %s,\n" % (kw, repr(val)))

    return '{\n' + string.join(kwlist, '') + '            }'

def build_bin(bin):
    bins = []
    for i in range(0, len(bin), 16):
        bins.append(bin[i:i+16])

    bins2 = []
    for i in range(0, len(bins), 2):
        try:
            bins2.append("'%s' '%s'" % (bins[i], bins[i + 1]))
        except IndexError:
            bins2.append("'%s'" % bins[i])

    return string.join(bins2, ' \\\n            ')


request_var_defs = {
    'InternAtom': ('fuzzy_prop', ),
    'ChangeProperty': [((8, ''), ),
                       ((8, 'foo'), ),
                       ((8, 'zoom'), ),
                       ((16, []), ),
                       ((16, [1, 2, 3]), ),
                       ((16, [1, 2, 3, 4]), ),
                       ((32, []), ),
                       ((32, [1, 2, 3]), ) ],
    'OpenFont': ('foofont', ),
    'QueryTextExtents': ('foo', ),
    'ListFonts': ('bhazr', ),
    'ListFontsWithInfo': ('bhazr2', ),
    'SetFontPath': [(['foo', 'bar', 'gazonk'], ),
                    ([], ) ],
    'SetDashes': (9, ),
    'SetClipRectangles': [(('Rectangle', 2), ),
                          (('Rectangle', 0), ) ],
    'PolyPoint': (('Point', 3), ),
    'PolyLine': (('Point', 5), ),
    'PolySegment': (('Segment', 1), ),
    'PolyRectangle': (('Rectangle', 3), ),
    'PolyArc': (('Arc', 3), ),
    'FillPoly': (('Point', 3), ),
    'PolyFillRectangle': (('Rectangle', 2), ),
    'PolyFillArc': (('Arc', 1), ),
    'PutImage': ('bit map data', ),
    'ImageText8': ('showme', ),
    'ImageText16': ('showmore', ),
    'AllocNamedColor': ('octarin', ),
    'FreeColors': (17, ),
    'StoreColors': (('ColorItem', 4), ),
    'StoreNamedColor': ('blue', ),
    'QueryColors': [(8, ), (0, ) ],
    'LookupColor': ('octarin', ),
    'QueryExtension': ('XTRA', ),
    'ChangeHosts': (4, ),
    'RotateProperties': (12, ),
    'SetPointerMapping': (5, ),
    }

reply_var_defs = {
    'QueryTree': (7, ),
    'GetAtomName': ('WM_CLASS', ),
    'GetProperty': [((8, ''), ),
                       ((8, 'foo'), ),
                       ((8, 'zoom'), ),
                       ((16, []), ),
                       ((16, [1, 2, 3]), ),
                       ((16, [1, 2, 3, 4]), ),
                       ((32, []), ),
                       ((32, [1, 2, 3]), ) ],
    'ListProperties': (23, ),
    'GetMotionEvents': (('Timecoord', 5), ),
    'QueryKeymap': (32, ),
    'QueryFont': (('FontProp', 1), ('CharInfo', 3)),
    'ListFonts': (['fie', 'fuzzy', 'foozooom'], ),
    'ListFontsWithInfo': (('FontProp', 1), 'fontfont'),
    'GetFontPath': [(['path1', 'path2232'], ),
                    ([], ) ],
    'GetImage': ('this is real ly imag e b-map', ),
    'ListInstalledColormaps': (2, ),
    'AllocColorCells': [(17, 3),
                        (0, 0) ],
    'AllocColorPlanes': (4, ),
    'QueryColors': (('rgb', 5), ),
    'ListExtensions': (['XTRA', 'XTRA-II'], ),
    'GetKeyboardControl': (32, ),
    'GetPointerMapping': (5, ),
#    '': (, ),
    }

event_var_defs = {
    'ClientMessage': [((8, '01234567890123456789'), ),
                      ((16, [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]), ),
                      ((32, [1, 2, 3, 4, 5]), ) ],
    }

structcode_defs = {
    'b': ('INT8', -128, -1, 1),
    'B': ('CARD8', 128, 255, 1),
    'h': ('INT16', -32768, -1, 2),
    'H': ('CARD16', 32768, 65536, 2),
    'l': ('INT32', -2147483647, -1, 4),
    'L': ('CARD32', 65536, 2147483646, 4),
    }

C_HEADER = r'''
#include <stdio.h>
#include <assert.h>
#include <ctype.h>
#include <X11/Xproto.h>

void output(char *name, void *data, int length)
{
    unsigned char *d = (unsigned char *) data;

    printf("%s ", name);

    while (length-- > 0) {
        if (0 && isgraph(*d))
            putchar(*d);
        else
            printf("\\x%02x", *d);
        d++;
    }

    putchar('\n');
}

'''

PY_HEADER = r'''#!/usr/bin/env python

import sys, os
sys.path.insert(1, os.path.join(sys.path[0], '..'))

import string
import unittest
from Xlib.protocol import request, rq, event
import Xlib.protocol.event

import struct
import array

class CmpArray:
    def __init__(self, *args, **kws):
        self.array = apply(array.array, args, kws)

    def __len__(self):
        return len(self.array)

    def __getslice__(self, x, y):
        return list(self.array[x:y])

    def __getattr__(self, attr):
        return getattr(self.array, attr)

    def __cmp__(self, other):
        return cmp(self.array.tolist(), other)

rq.array = CmpArray

def tohex(bin):
    bin = string.join(map(lambda c: '\\x%%02x' %% ord(c), bin), '')

    bins = []
    for i in range(0, len(bin), 16):
        bins.append(bin[i:i+16])

    bins2 = []
    for i in range(0, len(bins), 2):
        try:
            bins2.append("'%%s' '%%s'" %% (bins[i], bins[i + 1]))
        except IndexError:
            bins2.append("'%%s'" %% bins[i])

    return string.join(bins2, ' \\\n            ')

class DummyDisplay:
    def get_resource_class(self, x):
        return None

    event_classes = Xlib.protocol.event.event_class
dummy_display = DummyDisplay()


def check_endian():
    if struct.unpack('BB', struct.pack('H', 0x0100))[0] != %(endvalue)d:
        sys.stderr.write('%(endname)s tests, skipping on this system.\n')
        sys.exit(0)

'''

if __name__ == '__main__':
    build()



#!/usr/bin/env python

import sys, os
sys.path.insert(1, os.path.join(sys.path[0], '..'))

import string
import unittest
from Xlib.protocol import request, rq, event
import Xlib.protocol.event

import struct
import array

class CmpArray:
    def __init__(self, *args, **kws):
        self.array = apply(array.array, args, kws)

    def __len__(self):
        return len(self.array)

    def __getslice__(self, x, y):
        return list(self.array[x:y])

    def __getattr__(self, attr):
        return getattr(self.array, attr)

    def __cmp__(self, other):
        return cmp(self.array.tolist(), other)

rq.array = CmpArray

def tohex(bin):
    bin = string.join(map(lambda c: '\\x%02x' % ord(c), bin), '')

    bins = []
    for i in range(0, len(bin), 16):
        bins.append(bin[i:i+16])

    bins2 = []
    for i in range(0, len(bins), 2):
        try:
            bins2.append("'%s' '%s'" % (bins[i], bins[i + 1]))
        except IndexError:
            bins2.append("'%s'" % bins[i])

    return string.join(bins2, ' \\\n            ')

class DummyDisplay:
    def get_resource_class(self, x):
        return None

    event_classes = Xlib.protocol.event.event_class
dummy_display = DummyDisplay()


def check_endian():
    if struct.unpack('BB', struct.pack('H', 0x0100))[0] != 0:
        sys.stderr.write('Little-endian tests, skipping on this system.\n')
        sys.exit(0)



class TestKeymapNotify(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'type': 173,
            'data': [130, 181, 177, 244, 167, 144, 216, 185, 228, 220, 254, 130, 239, 213, 142, 240, 233, 248, 161, 238, 160, 205, 212, 205, 166, 156, 241, 169, 198, 147, 144],
            }
        self.evt_bin_0 = '\xad\x82\xb5\xb1' '\xf4\xa7\x90\xd8' \
            '\xb9\xe4\xdc\xfe' '\x82\xef\xd5\x8e' \
            '\xf0\xe9\xf8\xa1' '\xee\xa0\xcd\xd4' \
            '\xcd\xa6\x9c\xf1' '\xa9\xc6\x93\x90'


    def testPack0(self):
        bin = apply(event.KeymapNotify._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.KeymapNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestExpose(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'count': 31063,
            'width': 57024,
            'window': 1993119152,
            'y': 29154,
            'x': 15652,
            'type': 192,
            'sequence_number': 45668,
            'height': 29709,
            }
        self.evt_bin_0 = '\xc0\x00\x64\xb2' '\xb0\x95\xcc\x76' \
            '\x24\x3d\xe2\x71' '\xc0\xde\x0d\x74' \
            '\x57\x79\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.Expose._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.Expose._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestGraphicsExpose(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'count': 7002,
            'width': 21650,
            'major_event': 238,
            'minor_event': 44368,
            'y': 2412,
            'x': 50041,
            'drawable': 950531249,
            'type': 138,
            'sequence_number': 9516,
            'height': 10465,
            }
        self.evt_bin_0 = '\x8a\x00\x2c\x25' '\xb1\xf4\xa7\x38' \
            '\x79\xc3\x6c\x09' '\x92\x54\xe1\x28' \
            '\x50\xad\x5a\x1b' '\xee\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.GraphicsExpose._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.GraphicsExpose._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestNoExpose(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'minor_event': 49058,
            'window': 1389793826,
            'type': 198,
            'major_event': 149,
            'sequence_number': 51301,
            }
        self.evt_bin_0 = '\xc6\x00\x65\xc8' '\x22\x92\xd6\x52' \
            '\xa2\xbf\x95\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.NoExpose._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.NoExpose._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestVisibilityNotify(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'window': 848538738,
            'type': 233,
            'state': 239,
            'sequence_number': 38248,
            }
        self.evt_bin_0 = '\xe9\x00\x68\x95' '\x72\xac\x93\x32' \
            '\xef\x00\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.VisibilityNotify._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.VisibilityNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestCreateNotify(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'width': 24559,
            'window': 1328254552,
            'parent': 112487253,
            'override': 0,
            'y': -31372,
            'x': -13676,
            'border_width': 32812,
            'type': 230,
            'sequence_number': 14268,
            'height': 8803,
            }
        self.evt_bin_0 = '\xe6\x00\xbc\x37' '\x55\x6b\xb4\x06' \
            '\x58\x8e\x2b\x4f' '\x94\xca\x74\x85' \
            '\xef\x5f\x63\x22' '\x2c\x80\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.CreateNotify._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.CreateNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestDestroyNotify(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'window': 1384567865,
            'type': 183,
            'event': 1596763581,
            'sequence_number': 37839,
            }
        self.evt_bin_0 = '\xb7\x00\xcf\x93' '\xbd\xad\x2c\x5f' \
            '\x39\xd4\x86\x52' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.DestroyNotify._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.DestroyNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestUnmapNotify(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'window': 1267184116,
            'type': 192,
            'event': 913541146,
            'sequence_number': 55135,
            'from_configure': 0,
            }
        self.evt_bin_0 = '\xc0\x00\x5f\xd7' '\x1a\x88\x73\x36' \
            '\xf4\xb1\x87\x4b' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.UnmapNotify._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.UnmapNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestMapNotify(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'override': 0,
            'window': 2002432488,
            'type': 216,
            'event': 1566597012,
            'sequence_number': 8920,
            }
        self.evt_bin_0 = '\xd8\x00\xd8\x22' '\x94\x5f\x60\x5d' \
            '\xe8\xb1\x5a\x77' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.MapNotify._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.MapNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestMapRequest(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'window': 1740270413,
            'type': 242,
            'parent': 1188866605,
            'sequence_number': 6729,
            }
        self.evt_bin_0 = '\xf2\x00\x49\x1a' '\x2d\xaa\xdc\x46' \
            '\x4d\x6b\xba\x67' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.MapRequest._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.MapRequest._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestReparentNotify(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'override': 0,
            'window': 918878719,
            'parent': 1046822430,
            'y': -10755,
            'x': -11814,
            'type': 185,
            'event': 1344092894,
            'sequence_number': 31034,
            }
        self.evt_bin_0 = '\xb9\x00\x3a\x79' '\xde\x3a\x1d\x50' \
            '\xff\xf9\xc4\x36' '\x1e\x3e\x65\x3e' \
            '\xda\xd1\xfd\xd5' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.ReparentNotify._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.ReparentNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestConfigureNotify(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'y': -32583,
            'above_sibling': 1143940649,
            'height': 44365,
            'width': 24191,
            'window': 1699527401,
            'override': 1,
            'x': -23713,
            'border_width': 51797,
            'type': 191,
            'event': 2102634753,
            'sequence_number': 21818,
            }
        self.evt_bin_0 = '\xbf\x00\x3a\x55' '\x01\xa9\x53\x7d' \
            '\xe9\xba\x4c\x65' '\x29\x26\x2f\x44' \
            '\x5f\xa3\xb9\x80' '\x7f\x5e\x4d\xad' \
            '\x55\xca\x01\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.ConfigureNotify._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.ConfigureNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestConfigureRequest(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'parent': 1484835068,
            'width': 46666,
            'value_mask': 41755,
            'stack_mode': 155,
            'height': 27280,
            'sibling': 1153557246,
            'window': 549283037,
            'y': -1019,
            'x': -11524,
            'border_width': 41299,
            'type': 140,
            'sequence_number': 48820,
            }
        self.evt_bin_0 = '\x8c\x9b\xb4\xbe' '\xfc\xc8\x80\x58' \
            '\xdd\x64\xbd\x20' '\xfe\xe2\xc1\x44' \
            '\xfc\xd2\x05\xfc' '\x4a\xb6\x90\x6a' \
            '\x53\xa1\x1b\xa3' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.ConfigureRequest._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.ConfigureRequest._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestGravityNotify(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'window': 1471159096,
            'y': -26841,
            'x': -10882,
            'type': 191,
            'event': 860169186,
            'sequence_number': 48472,
            }
        self.evt_bin_0 = '\xbf\x00\x58\xbd' '\xe2\x23\x45\x33' \
            '\x38\x1b\xb0\x57' '\x7e\xd5\x27\x97' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.GravityNotify._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.GravityNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestResizeRequest(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'width': 8842,
            'window': 995086195,
            'type': 139,
            'sequence_number': 9443,
            'height': 58942,
            }
        self.evt_bin_0 = '\x8b\x00\xe3\x24' '\x73\xcf\x4f\x3b' \
            '\x8a\x22\x3e\xe6' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.ResizeRequest._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.ResizeRequest._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestPropertyNotify(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'window': 1763395006,
            'time': 936540618,
            'atom': 47197280,
            'type': 205,
            'state': 241,
            'sequence_number': 47586,
            }
        self.evt_bin_0 = '\xcd\x00\xe2\xb9' '\xbe\x45\x1b\x69' \
            '\x60\x2c\xd0\x02' '\xca\x79\xd2\x37' \
            '\xf1\x00\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.PropertyNotify._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.PropertyNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestSelectionClear(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'window': 336291153,
            'atom': 256452607,
            'type': 232,
            'sequence_number': 26660,
            'time': 1732839301,
            }
        self.evt_bin_0 = '\xe8\x00\x24\x68' '\x85\x07\x49\x67' \
            '\x51\x65\x0b\x14' '\xff\x27\x49\x0f' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.SelectionClear._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.SelectionClear._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestSelectionRequest(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'requestor': 264947265,
            'selection': 1535909824,
            'target': 607705863,
            'time': 1423586793,
            'owner': 764886771,
            'property': 1148098854,
            'type': 147,
            'sequence_number': 20571,
            }
        self.evt_bin_0 = '\x93\x00\x5b\x50' '\xe9\x35\xda\x54' \
            '\xf3\x3e\x97\x2d' '\x41\xc6\xca\x0f' \
            '\xc0\x1f\x8c\x5b' '\x07\xdb\x38\x24' \
            '\x26\x99\x6e\x44' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.SelectionRequest._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.SelectionRequest._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestSelectionNotify(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'requestor': 971528625,
            'selection': 327380230,
            'target': 1874329297,
            'time': 1022248107,
            'property': 1791820478,
            'type': 133,
            'sequence_number': 30741,
            }
        self.evt_bin_0 = '\x85\x00\x15\x78' '\xab\x44\xee\x3c' \
            '\xb1\x59\xe8\x39' '\x06\x6d\x83\x13' \
            '\xd1\xfe\xb7\x6f' '\xbe\x02\xcd\x6a' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.SelectionNotify._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.SelectionNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestColormapNotify(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'window': 1353796539,
            'colormap': 659729309,
            'new': 1,
            'type': 211,
            'state': 168,
            'sequence_number': 8684,
            }
        self.evt_bin_0 = '\xd3\x00\xec\x21' '\xbb\x4b\xb1\x50' \
            '\x9d\xab\x52\x27' '\x01\xa8\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.ColormapNotify._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.ColormapNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


class TestClientMessage(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'type': 237,
            'window': 1804643202,
            'client_type': 455293257,
            'data': (8, '01234567890123456789'),
            'sequence_number': 14854,
            }
        self.evt_bin_0 = '\xed\x08\x06\x3a' '\x82\xab\x90\x6b' \
            '\x49\x39\x23\x1b' '\x30\x31\x32\x33' \
            '\x34\x35\x36\x37' '\x38\x39\x30\x31' \
            '\x32\x33\x34\x35' '\x36\x37\x38\x39'

        self.evt_args_1 = {
            'type': 160,
            'window': 948875838,
            'client_type': 212297388,
            'data': (16, [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]),
            'sequence_number': 28171,
            }
        self.evt_bin_1 = '\xa0\x10\x0b\x6e' '\x3e\xb2\x8e\x38' \
            '\xac\x66\xa7\x0c' '\x01\x00\x02\x00' \
            '\x03\x00\x04\x00' '\x05\x00\x06\x00' \
            '\x07\x00\x08\x00' '\x09\x00\x0a\x00'

        self.evt_args_2 = {
            'type': 243,
            'window': 581929030,
            'client_type': 966878718,
            'data': (32, [1, 2, 3, 4, 5]),
            'sequence_number': 63569,
            }
        self.evt_bin_2 = '\xf3\x20\x51\xf8' '\x46\x88\xaf\x22' \
            '\xfe\x65\xa1\x39' '\x01\x00\x00\x00' \
            '\x02\x00\x00\x00' '\x03\x00\x00\x00' \
            '\x04\x00\x00\x00' '\x05\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.ClientMessage._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.ClientMessage._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)

    def testPack1(self):
        bin = apply(event.ClientMessage._fields.to_binary, (), self.evt_args_1)
        try:
            assert bin == self.evt_bin_1
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack1(self):
        args, remain = event.ClientMessage._fields.parse_binary(self.evt_bin_1, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_1
        except AssertionError:
            raise AssertionError(args)

    def testPack2(self):
        bin = apply(event.ClientMessage._fields.to_binary, (), self.evt_args_2)
        try:
            assert bin == self.evt_bin_2
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack2(self):
        args, remain = event.ClientMessage._fields.parse_binary(self.evt_bin_2, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_2
        except AssertionError:
            raise AssertionError(args)


class TestMappingNotify(unittest.TestCase):
    def setUp(self):
        self.evt_args_0 = {
            'first_keycode': 246,
            'request': 189,
            'type': 198,
            'count': 201,
            'sequence_number': 32665,
            }
        self.evt_bin_0 = '\xc6\x00\x99\x7f' '\xbd\xf6\xc9\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00' \
            '\x00\x00\x00\x00' '\x00\x00\x00\x00'


    def testPack0(self):
        bin = apply(event.MappingNotify._fields.to_binary, (), self.evt_args_0)
        try:
            assert bin == self.evt_bin_0
        except AssertionError:
            raise AssertionError(tohex(bin))

    def testUnpack0(self):
        args, remain = event.MappingNotify._fields.parse_binary(self.evt_bin_0, dummy_display, 1)
        try:
            assert len(remain) == 0
        except AssertionError:
            raise AssertionError(tohex(remain))
        try:
            assert args == self.evt_args_0
        except AssertionError:
            raise AssertionError(args)


if __name__ == "__main__":
    check_endian()
    unittest.main()



#!/usr/bin/python
# $Id: threadtest.py,v 1.3 2007/06/10 14:11:59 mggrant Exp $

import sys
import os

sys.path[1:1] = [os.path.join(sys.path[0], '..')]

from Xlib import display, X, threaded
import time
import thread

def redraw(win, gc):
    # win.clear_area()
    win.fill_rectangle(gc, 20, 20, 60, 60)

def blink(display, win, gc, cols):
    while 1:
        time.sleep(2)
        print 'Changing color', cols[0]
        gc.change(foreground = cols[0])
        cols = (cols[1], cols[0])
        redraw(win, gc)
        display.flush()

def main():
    d = display.Display()
    root = d.screen().root

    colormap = d.screen().default_colormap

    red = colormap.alloc_named_color("red").pixel
    blue = colormap.alloc_named_color("blue").pixel
    background = colormap.alloc_named_color("white").pixel

    window = root.create_window(100, 100, 100, 100, 1,
                                X.CopyFromParent, X.InputOutput,
                                X.CopyFromParent,
                                background_pixel = background,
                                event_mask = X.StructureNotifyMask | X.ExposureMask)
    window.map()

    gc = window.create_gc(foreground = red)

    thread.start_new_thread(blink, (d, window, gc, (blue, red)))

    while 1:
        event = d.next_event()
        if event.type == X.Expose:
            if event.count == 0:
                redraw(window, gc)
        elif event.type == X.DestroyNotify:
            sys.exit(0)

if __name__ == "__main__":
    main()



#!/usr/bin/python

import socket
import select
import sys
import os

def tcpbug(listenaddr, destaddr, outputfd):
    listen = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    listen.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    listen.bind(listenaddr)
    listen.listen(1)

    client, addr = listen.accept()

    server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server.connect(destaddr)

    cwbuf = swbuf = outbuf = ''

    while client or server or outbuf:
        readset = []
        if client:
            readset.append(client)
        if server:
            readset.append(server)

        writeset = []
        if cwbuf and client:
            writeset.append(client)
        if swbuf and server:
            writeset.append(server)
        if outbuf:
            writeset.append(outputfd)

        readset, writeset, e = select.select(readset, writeset, [])

        if client in readset:
            data = client.recv(2000)
            if data:
                swbuf = swbuf + data
                outbuf = outbuf + 'C%d\n%s' % (len(data), data)
            else:
                client = None

        if server in readset:
            data = server.recv(2000)
            if data:
                cwbuf = cwbuf + data
                outbuf = outbuf + 'S%d\n%s' % (len(data), data)
            else:
                server = None

        if client in writeset:
            n = client.send(cwbuf)
            cwbuf = cwbuf[n:]

        if server in writeset:
            n = server.send(swbuf)
            swbuf = swbuf[n:]

        if outputfd in writeset:
            n = os.write(outputfd, outbuf)
            outbuf = outbuf[n:]

        if client and not cwbuf and not server:
            client.close()
            client = None

        if server and not swbuf and not client:
            server.close()
            server = None

def main():
    try:
        lport = int(sys.argv[1])
        dhost = sys.argv[2]
        dport = int(sys.argv[3])
    except (IndexError, ValueError):
        sys.stderr.write('Usage: %s listenport desthost destport\n'
                         % sys.argv[0])
        exit(1)

    tcpbug(('', lport), (dhost, dport), 1)

if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# $Id: shapewin.py,v 1.2 2007/06/10 14:11:59 mggrant Exp $
#
# examples/shapewin.py -- demonstrate shape extension
#
#    Copyright (C) 2002 Peter Liljenberg <petli@ctrl-c.liu.se>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import sys
import os

# Change path so we find Xlib
sys.path.insert(1, os.path.join(sys.path[0], '..'))

from Xlib import X, display, Xutil
from Xlib.ext import shape

# Application window (only one)
class Window:
    def __init__(self, display):
        self.d = display

        # Check for extension
        if not self.d.has_extension('SHAPE'):
            sys.stderr.write('%s: server does not have SHAPE extension\n'
                             % sys.argv[1])
            sys.exit(1)

        # print version
        r = self.d.shape_query_version()
        print 'SHAPE version %d.%d' % (r.major_version, r.minor_version)


        # Find which screen to open the window on
        self.screen = self.d.screen()

        # background pattern
        bgsize = 20

        bgpm = self.screen.root.create_pixmap(bgsize, bgsize, self.screen.root_depth)

        bggc = self.screen.root.create_gc(foreground = self.screen.black_pixel,
                                          background = self.screen.black_pixel)

        bgpm.fill_rectangle(bggc, 0, 0, bgsize, bgsize)

        bggc.change(foreground = self.screen.white_pixel)

        bgpm.arc(bggc, -bgsize / 2, 0, bgsize, bgsize, 0, 360 * 64)
        bgpm.arc(bggc, bgsize / 2, 0, bgsize, bgsize, 0, 360 * 64)
        bgpm.arc(bggc, 0, -bgsize / 2, bgsize, bgsize, 0, 360 * 64)
        bgpm.arc(bggc, 0, bgsize / 2, bgsize, bgsize, 0, 360 * 64)

        # Actual window
        self.window = self.screen.root.create_window(
            100, 100, 400, 300, 0,
            self.screen.root_depth,
            X.InputOutput,
            X.CopyFromParent,

            # special attribute values
            background_pixmap = bgpm,
            event_mask = (X.StructureNotifyMask |
                          X.ButtonReleaseMask),
            colormap = X.CopyFromParent,
            )

        # Set some WM info

        self.WM_DELETE_WINDOW = self.d.intern_atom('WM_DELETE_WINDOW')
        self.WM_PROTOCOLS = self.d.intern_atom('WM_PROTOCOLS')

        self.window.set_wm_name('Xlib example: shapewin.py')
        self.window.set_wm_icon_name('shapewin.py')
        self.window.set_wm_class('shapewin', 'XlibExample')

        self.window.set_wm_protocols([self.WM_DELETE_WINDOW])
        self.window.set_wm_hints(flags = Xutil.StateHint,
                                 initial_state = Xutil.NormalState)

        self.window.set_wm_normal_hints(flags = (Xutil.PPosition | Xutil.PSize
                                                 | Xutil.PMinSize),
                                        min_width = 50,
                                        min_height = 50)

        # The add and subtract shapes
        self.add_size = 60

        self.add_pm = self.window.create_pixmap(self.add_size, self.add_size, 1)
        gc = self.add_pm.create_gc(foreground = 0, background = 0)
        self.add_pm.fill_rectangle(gc, 0, 0, self.add_size, self.add_size)
        gc.change(foreground = 1)
        self.add_pm.fill_arc(gc, 0, 0, self.add_size, self.add_size, 0, 360 * 64)
        gc.free()

        self.sub_size = 59
        self.sub_pm = self.window.create_pixmap(self.sub_size, self.sub_size, 1)
        gc = self.sub_pm.create_gc(foreground = 0, background = 0)
        self.sub_pm.fill_rectangle(gc, 0, 0, self.sub_size, self.sub_size)
        gc.change(foreground = 1)
        self.sub_pm.fill_poly(gc, X.Convex, X.CoordModeOrigin,
                              [(self.sub_size / 2, 0),
                               (self.sub_size, self.sub_size / 2),
                               (self.sub_size / 2, self.sub_size),
                               (0, self.sub_size / 2)])
        gc.free()

        # Set initial mask
        self.window.shape_mask(shape.ShapeSet, shape.ShapeBounding,
                               0, 0, self.add_pm)
        self.window.shape_mask(shape.ShapeUnion, shape.ShapeBounding,
                               400 - self.add_size, 0, self.add_pm)
        self.window.shape_mask(shape.ShapeUnion, shape.ShapeBounding,
                               0, 300 - self.add_size, self.add_pm)
        self.window.shape_mask(shape.ShapeUnion, shape.ShapeBounding,
                               400 - self.add_size, 300 - self.add_size,
                               self.add_pm)

        # Tell X server to send us mask events
        self.window.shape_select_input(1)

        # Map the window, making it visible
        self.window.map()


    # Main loop, handling events
    def loop(self):
        current = None
        while 1:
            e = self.d.next_event()

            # Window has been destroyed, quit
            if e.type == X.DestroyNotify:
                sys.exit(0)

            # Button released, add or subtract
            elif e.type == X.ButtonRelease:
                if e.detail == 1:
                    self.window.shape_mask(shape.ShapeUnion,
                                           shape.ShapeBounding,
                                           e.event_x - self.add_size / 2,
                                           e.event_y - self.add_size / 2,
                                           self.add_pm)
                elif e.detail == 3:
                    self.window.shape_mask(shape.ShapeSubtract,
                                           shape.ShapeBounding,
                                           e.event_x - self.sub_size / 2,
                                           e.event_y - self.sub_size / 2,
                                           self.sub_pm)

            # Shape has changed
            elif e.type == self.d.extension_event.ShapeNotify:
                print 'Shape change'

            # Somebody wants to tell us something
            elif e.type == X.ClientMessage:
                if e.client_type == self.WM_PROTOCOLS:
                    fmt, data = e.data
                    if fmt == 32 and data[0] == self.WM_DELETE_WINDOW:
                        sys.exit(0)


if __name__ == '__main__':
    Window(display.Display()).loop()
